{% extends "password_reset/base.html" %}
{% load i18n %}

{% block title %}{% trans "Password recovery" %}{% endblock %}

{% block content %}
	<form method="post" action="{{ url }}">
		{% csrf_token %}
		{{ form.as_p }}
		<p><input type="submit" value="{% trans "Recover my password" %}"></p>
	</form>
{% endblock %}
